@@ -104,11 +104,32 @@ gok_discard_focus_filter (GdkXEvent *xevent,
 			  gpointer  data)
 {
   XEvent *xev = (XEvent *)xevent;	
+  GList *list, *l;
+  gint n_mapped;
+  GtkWindow *window;
+  gboolean has_focus;
 
   if (xev->xany.type == ClientMessage &&
       (Atom) xev->xclient.data.l[0] == gdk_x11_atom_to_xatom (
 	      gdk_atom_intern ("WM_TAKE_FOCUS", False)))
     {
+      list = gtk_window_list_toplevels ();
+      n_mapped = 0;
+      has_focus = FALSE;
+      for (l = list; l; l = l->next)
+        {
+          if (GTK_WIDGET_MAPPED (GTK_WIDGET (l->data)))
+            {
+              n_mapped++;
+              window = GTK_WINDOW (l->data);
+              if (window->has_focus)
+                has_focus = TRUE;
+            }
+        }
+      if (!has_focus && n_mapped > 1)
+        XSetInputFocus (xev->xany.display, GDK_WINDOW_XID (GTK_WIDGET (window)->window), RevertToPointerRoot, CurrentTime);
+
+      g_list_free (list);
       return GDK_FILTER_REMOVE;
     }
   else
@@ -122,8 +143,9 @@ on_window1_realize                     (GtkWidget       *widget,
                                         gpointer         user_data)
 {
   XWMHints wm_hints;
-  Atom wm_window_protocols[3];
+  Atom wm_window_protocols[3], atom_type_dock[1], atom_window_type;
   static gboolean initialized = FALSE;
+  gboolean is_dock = (gboolean) user_data;
   
   if (!initialized)
     {
@@ -135,11 +157,26 @@ on_window1_realize                     (GtkWidget       *widget,
   wm_hints.flags = InputHint;
   wm_hints.input = False;
   
+  gdk_error_trap_push ();
+
   XSetWMHints (GDK_WINDOW_XDISPLAY (widget->window),
 	       GDK_WINDOW_XWINDOW (widget->window), &wm_hints);
   
   XSetWMProtocols (GDK_WINDOW_XDISPLAY (widget->window),
 		   GDK_WINDOW_XWINDOW (widget->window), wm_window_protocols, 3);
+  gdk_error_trap_pop ();
+  
+  if (is_dock) {
+	  gok_main_set_wm_dock (TRUE);
+  }
+  else {
+	  if (widget->window) {
+		  gdk_window_set_decorations (widget->window, GDK_DECOR_ALL | 
+					      GDK_DECOR_MINIMIZE | GDK_DECOR_MAXIMIZE);
+		  gdk_window_set_functions (widget->window, GDK_FUNC_MOVE | GDK_FUNC_RESIZE);
+	  }
+  }
+
   gtk_window_stick (GTK_WINDOW (widget));
 
   if (!gok_keyboard_xkb_select (GDK_WINDOW_XDISPLAY (widget->window)))
@@ -154,6 +191,7 @@ on_window1_realize                     (GtkWidget       *widget,
   gdk_window_add_filter (widget->window,
 			 gok_discard_focus_filter,
 			 NULL);
+
 }
 
 
@@ -929,7 +967,3 @@ on_joystick_button_toggled             (GtkToggleButton *togglebutton,
 	gok_page_actions_set_is_corepointer (
 		!gtk_toggle_button_get_active (togglebutton));
 }
-
-
-
-
